/*
    capping.js : Gre le capping des publicits.

    Lors du chargement d'une pub, le navigateur mmorise des infos de capping,
    de manire  pouvoir les transmettre au serveur lors des slections de pubs
    suivantes :
    * l'autohandler charge ce capping.js, qui contient une grosse fonction enregistre_capping()
    * chaque pub devant tre cappe doit appeler cette fonction, avec des paramtres ( voir dans une pub normale cappe, et reproduire dans les pub.js ou walpaper.js hacked)
    * cet appel JS dans la navigateur mmorise le nombre d'affichage de chaque pub cappe
    *  chaque demande de pub ultrieure, le navigateur envoie les infos de capping  e-pubs/pub.phtml
    * ce dernier utilise les infos de capping pour exclure les pubs arrives au maxi

    Attention : IE interprte le script avec le mme encodage que la page, soit UTF8,
    mais comme le fichier est crit en Latin1, tous les accents sont casss, et
    certains bouffent les \n, rendant la syntaxe du script errone. Quand la page n'a
    pas d'encodage, elle est considre comme Latin1, et le mme script passe bien...
*/

// enregistre_capping : enregistre un nouvel affichage d'une pub, et ajuste les compteurs de capping (cookie)
//  cle_pub : cl de la pb affiche (pas d'accent en fin de ligne, IE est trop c.n)
//  nombre_max : nombre d'affichages autoriss (capping_partiel)
//  periode_max : priode sur laquelle le nombre s'applique (en jours) (capping_total)
// (les noms entre parenthse correspondent  ce qui est stock dans la base de donnes, qui ne contient que ces donnes, et pas de priode)
// Le processus est le suivant :
//  - enregistre le nouvel affichage de la cle_pub, avec sa date, dans un cookie
//  - supprime les anciens affichages (plus vieux que la priode et plus nombreux que le nombre d'affichages)
//  - le cookie sera envoy au serveur  la prochaine demande de pub
//  - le serveur tiendra compte des cle_pub  exclure, en fonction du nombre d'affichage, de la priode et des dates
// C'est le serveur qui doit mesurer les exclusions, au moment du prochain affichage, car il peut se trouver loin dans le futur.
// On peut se retrouver, lors de l'excution de enregistre_capping(),  arriver au nombre maxi, mais le prochain affichage ne se fera
// peut-tre qu'aprs expiration de la priode, et donc la pub pourra tout de mme tre affiche.

function enregistre_capping(cle_pub, nombre_max, periode_max) {
    //alert('enregistre_capping('+cle_pub+','+nombre_max+','+periode_max+')');
    
    // Dtermine la date actuelle de l'affichage,  mmoriser
    var date_courante = Math.round((new Date()).getTime() / 1000); // getTime() renvoie des millimes de seconde, mais on veut des secondes coules depuis le 1/1/1970...

    // Rcupre la valeur de l'ancien cookie...
    // Les cookies ont la forme suivante, encod en JSON (http://json.org) :
    //  {cle:[nombre_max,periode_max,[time,time...],cle:[...]}
    var cookie = document.cookie.match(/(?:^|; )CAP=([^;]*)/);
    var capping;
    try {
        // Dans le cas o le cookie n'est pas valuable, il faut continuer comme si de rien n'tait...
        //document.write('Evaluation de '+decodeURIComponent(cookie[1])+'<br>');
        capping = cookie ? eval("(" + decodeURIComponent(cookie[1]) + ")") : undefined;
    } finally {
        // Aprs valuation, si on n'a pas un Objet (hash contenant une cl par publicit cappe), on en initialise un vide
        // IE ne laisse pas un objet aprs le eval, comme Moz, donc il faut le recrer pour que le test un peu aprs passe correctement
        if (typeof(capping) != 'object') {
            capping = [];
        }
    }

    // On ajoute l'affichage en cours de la cle_pub demande (pas d'accent en fin de ligne, IE est trop c.n)
    if (!(cle_pub in capping)) {
        capping[cle_pub] = []; // ...dans le cas o elle n'existe pas encore, on cr le tableau la dcrivant
    }
    capping[cle_pub][0] = nombre_max; // ...puis on ajoute les variables de la pub
    capping[cle_pub][1] = periode_max;
    if (2 in capping[cle_pub]) {
        capping[cle_pub][2].push(date_courante); // S'il existe dj une date, on l'ajoute  la fin...
    } else {
        capping[cle_pub][2] = [date_courante]; // ...mais s'il n'y en a pas encore, on cre le tableau du les contiendra toutes.
    }

    // On parcoure toutes les cls stockes, et on ajuste leurs valeurs :
    //  - ne conserver que nombre_max dates 
    //  - supprimer les dates antrieures  periode_max
    //  - s'il ne reste aucune date, supprimer toute la cl (ne pas laisser d'accent en fin de ligne, IE n'aime pas a, quel c.n)
    for (var cle in capping) {
        //document.write('Capping de cle='+cle+'<br>');
        if (capping[cle][2].length > capping[cle][0]) {
            capping[cle][2].splice(0, capping[cle][2].length - capping[cle][0]); // supprime les premier lments, pour n'en garder que nombre_max
        }
        while (capping[cle][2][0] < date_courante - periode_max*86400) {
            capping[cle][2].shift(); // supprime le premier lment, trop ancien
        }
        if (capping[cle][2].length == 0) {
            delete capping[cle]; // supprime toute la cl, s'il ne reste plus aucune date...
        }
    }

    // On reconstitue le cookie  partir de l'objet capping
    var cles = [];
    for (var cle in capping) {
        // Le format est adapt  ce que le package Perl JSON (libjson-perl) est capable de dcoder : notamment, les identifiants sont placs entre " et non entre '
        // cf. http://json.org/
        cles.push('"' + cle + '":[' + capping[cle][0] + ',' + capping[cle][1] + ',[' + capping[cle][2] + ']]');
    }
    cookie = '{' + cles.join(',') + '}';
    
    // Stocke le cookie, qui sera envoy la prochaine fois au serveur.
    // Il expirera dans trs longtemps, de manire  ne pas perdre les infos de capping...
    document.cookie = 'CAP=' + encodeURIComponent(cookie) + '; expires=Tue, 19 Jan 2038 04:14:07 GMT; path=/';

    cookie += '';
    return cookie;
}
